package com.qmth.exam.reserve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.exam.reserve.bean.Constants;
import com.qmth.exam.reserve.bean.login.LoginUser;
import com.qmth.exam.reserve.bean.user.UserReq;
import com.qmth.exam.reserve.bean.user.UserSaveReq;
import com.qmth.exam.reserve.bean.user.UserVO;
import com.qmth.exam.reserve.dao.UserDao;
import com.qmth.exam.reserve.entity.UserEntity;
import com.qmth.exam.reserve.enums.Role;
import com.qmth.exam.reserve.service.UserService;
import com.qmth.exam.reserve.util.PageUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

@Service
public class UserServiceImpl extends ServiceImpl<UserDao, UserEntity> implements UserService {

    private static final Logger log = LoggerFactory.getLogger(UserServiceImpl.class);

    @Override
    public UserEntity findUserByLoginName(Long orgId, String loginName) {
        LambdaQueryWrapper<UserEntity> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(UserEntity::getOrgId, orgId);
        wrapper.eq(UserEntity::getLoginName, loginName);
        return baseMapper.selectOne(wrapper);
    }

    @Override
    public PageResult<UserVO> pageUser(UserReq req) {
        IPage<UserVO> page = baseMapper.pageUser(new Page<>(req.getPageNumber(), req.getPageSize()), req);
        page.getRecords().forEach(user -> {
            user.setRoleName(Role.getRoleName(user.getRoleCode()));
        });
        return PageUtil.of(page);
    }

    @Override
    public void saveUser(LoginUser user, UserSaveReq req) {
        checkUser(req);

        UserEntity userEntity = new UserEntity();
        userEntity.setOrgId(user.getOrgId());
        BeanUtils.copyProperties(req, userEntity);

        if (req.getId() == null) {
            userEntity.setEnable(Boolean.TRUE);
            userEntity.setPassword(DigestUtils.sha256Hex(Constants.DEFAULT_PASSWORD).toUpperCase());
            this.save(userEntity);
        } else {
            this.updateById(userEntity);
        }
    }

    @Override
    public void resetPassword(Long id) {
        LambdaUpdateWrapper<UserEntity> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(UserEntity::getPassword, DigestUtils.sha256Hex(Constants.DEFAULT_PASSWORD).toUpperCase());
        updateWrapper.set(UserEntity::getFirstLogin, true);
        updateWrapper.set(UserEntity::getUpdateTime, System.currentTimeMillis());
        updateWrapper.eq(UserEntity::getId, id);

        this.update(updateWrapper);
        log.warn("重置用户密码！userId:{}", id);
    }

    @Override
    public void enable(Long id, Boolean enable) {
        LambdaUpdateWrapper<UserEntity> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(UserEntity::getEnable, enable);
        updateWrapper.set(UserEntity::getUpdateTime, System.currentTimeMillis());
        updateWrapper.eq(UserEntity::getId, id);

        this.update(updateWrapper);
        log.warn("更新用户状态！userId:{} enable:{}", id, enable);
    }

    private void checkUser(UserSaveReq req) {
        if (req.getRole() == null) {
            throw new StatusException("请选择角色");
        }
        if (StringUtils.isBlank(req.getLoginName())) {
            throw new StatusException("登录账号不能为空");
        }
        if (StringUtils.isBlank(req.getName())) {
            throw new StatusException("用户名称不能为空");
        }
        if (req.getRole().equals(Role.TEACHING) && req.getCategoryId() == null) {
            throw new StatusException("请选择所属教学点");
        }
    }

}
